from openfisca_france.model.base import Variable, Famille, MONTH, not_, max_, set_input_divide_by_period


class covid_aide_exceptionnelle_famille_montant(Variable):
    entity = Famille
    value_type = float
    label = "Montant de l'aide exceptionnelle pour les familles pendant la crise sanitaire dûe au COVID-19"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    end = '2020-10-31'
    reference = 'https://www.legifrance.gouv.fr/jorf/id/JORFTEXT000042574431'

    def formula_2020_05(famille, period, parameters):

        montants = parameters(period).prestations_sociales.solidarite_insertion.autre_solidarite.covid19.aide_exceptionnelle_famille
        rsa = famille('rsa', period) > 0
        ass = famille.sum(famille.members('ass', period)) > 0
        al = famille('aide_logement', period) > 0
        af_nbenf = famille('af_nbenf', period)
        aer = famille.sum(famille.members('aer', period)) > 0
        prim_forf = famille.sum(famille.members('prime_forfaitaire_mensuelle_reprise_activite', period)) > 0
        age_i = famille.members('age', period)
        etudiant_i = famille.members('etudiant', period)
        moins_de_25_ans_non_etudiant = famille.any((age_i <= 24) * (etudiant_i == 0), role = Famille.PARENT)
        base_jeune = moins_de_25_ans_non_etudiant * al
        base = (rsa + ass + aer + prim_forf)
        montant = (
            base_jeune * (montants.base_jeune + montants.par_enfant * af_nbenf)
            + base * not_(base_jeune) * (montants.base + montants.par_enfant * af_nbenf)
            + not_(base) * not_(base_jeune) * al * af_nbenf * montants.par_enfant
            )

        period_1 = period.offset(-1, 'month')
        rsa_n_1 = famille('rsa', period_1) > 0
        ass_n_1 = famille.sum(famille.members('ass', period_1)) > 0
        al_n_1 = famille('aide_logement', period_1) > 0
        af_nbenf_n_1 = famille('af_nbenf', period_1) > 0
        aer_n_1 = famille.sum(famille.members('aer', period_1)) > 0
        prim_forf_n_1 = famille.sum(famille.members('prime_forfaitaire_mensuelle_reprise_activite', period_1)) > 0
        age_i_n_1 = famille.members('age', period_1)
        etudiant_i_n_1 = famille.members('etudiant', period_1)
        moins_de_25_ans_non_etudiant_n_1 = famille.any((age_i_n_1 <= 24) * (etudiant_i_n_1 == 0), role = Famille.PARENT)
        base_jeune_n_1 = moins_de_25_ans_non_etudiant_n_1 * al_n_1
        base_n_1 = (rsa_n_1 + ass_n_1 + aer_n_1 + prim_forf_n_1)
        montant_n_1 = (
            base_jeune_n_1 * (montants.base_jeune + montants.par_enfant * af_nbenf_n_1)
            + base_n_1 * not_(base_jeune_n_1) * (montants.base + montants.par_enfant * af_nbenf_n_1)
            + not_(base_n_1) * not_(base_jeune_n_1) * al_n_1 * af_nbenf_n_1 * montants.par_enfant
            )

        return max_(montant, montant_n_1)



from .cache import tax_benefit_system


def test_metadata():
    metadata = tax_benefit_system.get_package_metadata()
    assert metadata['name'] == 'openfisca-france'
    assert metadata['repository_url'] == 'https://github.com/openfisca/openfisca-france'



import logging
from openfisca_france.model.base import *

log = logging.getLogger(__name__)


class assurance_vie_pl_non_anonyme_plus8ans_depuis1990(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Produits des bons ou contrats de capitalisation et d'assurance vie d'une durée d'au moins 8 ans pour les contrats souscrits depuis le 1er janvier 1990, et que le bénéficiaire décide de soumettre au prélèvement libératoire"
    definition_period = YEAR


class assurance_vie_pl_non_anonyme_plus6ans_avant1990(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Produits des bons ou contrats de capitalisation et d'assurance vie d'une durée d'au moins 6 ans pour les contrats souscrits précédemment, et que le bénéficiaire décide de soumettre au prélèvement libératoire"
    definition_period = YEAR


class assurance_vie_pl_non_anonyme_moins4ans_depuis1990(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Produits des bons ou contrats de capitalisation et d'assurance vie d'une durée de moins de 4 ans, pour les contrats souscrits depuis le 1er janvier 1990, et que le bénéficiaire décide de soumettre au prélèvement libératoire"
    definition_period = YEAR


class assurance_vie_pl_non_anonyme_4_8_ans_depuis1990(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Produits des bons ou contrats de capitalisation et d'assurance vie d'une durée de 4 à 8 ans, pour les contrats souscrits depuis le 1er janvier 1990, et que le bénéficiaire décide de soumettre au prélèvement libératoire"
    definition_period = YEAR


class assurance_vie_pl_anonyme(Variable):
    value_type = float
    entity = FoyerFiscal
    label = "Produits des bons ou contrats de capitalisation et d'assurance vie lorsque le bénéficiaire ne révèle pas son identité et son domicile fiscal, et qu'il décide de soumettre ces produits au prélèvement libératoire"
    definition_period = YEAR


class prelevement_forfaitaire_liberatoire(Variable):
    value_type = float
    entity = FoyerFiscal
    label = 'Prelèvement forfaitaire libératoire sur les revenus du capital'
    reference = [
        'Article 125-0 A du Code général des impôts',
        'https://www.legifrance.gouv.fr/codes/article_lc/LEGIARTI000006308140'
        ]
    definition_period = YEAR

    def formula_2002_01_01(foyer_fiscal, period, parameters):
        '''
        Prelèvement libératoire sur les revenus du capital
        '''
        f2dh = foyer_fiscal('f2dh', period)
        f2ee = foyer_fiscal('f2ee', period)
        param_pfl_av = parameters(period).taxation_capital.prelevement_forfaitaire.liberatoire_assurance_vie.bons_contrats_placements
        param_pfl = parameters(period).taxation_capital.prelevement_forfaitaire.liberatoire_taux_fixe

        return -(
            f2dh * param_pfl_av.apres_le_1_1_90.duree_8_ans_et_produits_apres_1998
            + f2ee * param_pfl.autres_produits_de_placement_a_revenu_fixe.courus_entre_le_1_1_95_et_apres
            )

    def formula_2008_01_01(foyer_fiscal, period, parameters):
        '''
        Prelèvement libératoire sur les revenus du capital
        '''
        f2da = foyer_fiscal('f2da', period)
        f2dh = foyer_fiscal('f2dh', period)
        f2ee = foyer_fiscal('f2ee', period)
        param_pfl_av = parameters(period).taxation_capital.prelevement_forfaitaire.liberatoire_assurance_vie.bons_contrats_placements
        param_pfl = parameters(period).taxation_capital.prelevement_forfaitaire.liberatoire_taux_fixe

        return -(
            f2da * param_pfl.dividendes
            + f2ee * param_pfl.autres_produits_de_placement_a_revenu_fixe.courus_entre_le_1_1_95_et_apres
            + f2dh * param_pfl_av.apres_le_1_1_90.duree_8_ans_et_produits_apres_1998
            )

    def formula_2013_01_01(foyer_fiscal, period, parameters):
        '''
        Prelèvement forfaitaire libératoire (PFL) sur les revenus du capital
        Notes :
          (1) La formule avant 2013 provient de l'ancienne variable 'imp_lib'.
              A priori, cette ancienne formule n'est pas entièrement correcte (application d'un taux unique à la case 2EE par exemple) et mériterait d'être corrigée.
          (2) Cette variable ne comprend pas la taxation forfaitaire à 24% associée
              à la base f2fa, qui est prise en compte via la variable tax_rvcm_forfaitaire,
              qui est incluse dans irpp. Cette taxaiton à 24% est certes forfaitaire,
              mais apparemment pas à la source. Elle serait calculée au moment de l'irpp,
              d'où le fait de l'inclure dans la variable IRPP.
        '''

        assurance_vie_pl_non_anonyme_plus8ans_depuis1990 = foyer_fiscal('assurance_vie_pl_non_anonyme_plus8ans_depuis1990', period)
        assurance_vie_pl_non_anonyme_plus6ans_avant1990 = foyer_fiscal('assurance_vie_pl_non_anonyme_plus6ans_avant1990', period)
        assurance_vie_pl_non_anonyme_moins4ans_depuis1990 = foyer_fiscal('assurance_vie_pl_non_anonyme_moins4ans_depuis1990', period)
        assurance_vie_pl_non_anonyme_4_8_ans_depuis1990 = foyer_fiscal('assurance_vie_pl_non_anonyme_4_8_ans_depuis1990', period)
        assurance_vie_pl_anonyme = foyer_fiscal('assurance_vie_pl_anonyme', period)
        produit_epargne_solidaire = foyer_fiscal('produit_epargne_solidaire', period)
        produit_etats_non_cooperatif = foyer_fiscal('produit_etats_non_cooperatif', period)

        param_pfl_av = parameters(period).taxation_capital.prelevement_forfaitaire.liberatoire_assurance_vie.bons_contrats_placements
        param_pfl = parameters(period).taxation_capital.prelevement_forfaitaire.liberatoire_taux_fixe

        pfl = -(
            (param_pfl_av.apres_le_1_1_90.duree_8_ans_et_produits_apres_1998
            * assurance_vie_pl_non_anonyme_plus8ans_depuis1990)
            + (param_pfl_av.entre_1_1_83_et_31_12_89.six_ans_produits_apres_98
            * assurance_vie_pl_non_anonyme_plus6ans_avant1990)
            + (param_pfl_av.apres_le_1_1_90.duree_moins_de_4_ans
            * assurance_vie_pl_non_anonyme_moins4ans_depuis1990)
            + (param_pfl_av.apres_le_1_1_90.duree_4_a_8_ans
            * assurance_vie_pl_non_anonyme_4_8_ans_depuis1990)
            + (param_pfl_av.avec_anonymat
            * assurance_vie_pl_anonyme)
            + (param_pfl.produits_epargne_solidaire_partage
            * produit_epargne_solidaire)
            + (param_pfl_av.produits_vers_etats_non_cooperatifs
            * produit_etats_non_cooperatif)
            )

        return pfl



from openfisca_france.model.base import *


DEFAULT_ROUND_BASE_DECIMALS = 2

cotisations_employeur_by_categorie_salarie = {
    'prive_cadre': [
        'agffc',
        'agirc_arrco',
        'agirc',
        'ags',
        'apec',
        'apprentissage_contribution_additionnelle',
        'apprentissage_taxe',
        'apprentissage_taxe_alsace_moselle',
        'arrco',
        'asf',
        'ceg',
        'cet',
        'cet2019',
        'chomage',
        'construction_plus_de_10_salaries',
        'construction_plus_de_20_salaries',
        'construction_plus_de_50_salaries',
        'csa',
        'famille',
        'financement_organisations_syndicales',
        'fnal_contribution_moins_de_20_salaries',
        'fnal_contribution_moins_de_50_salaries',
        'fnal_contribution_plus_de_10_salaries',
        'fnal_contribution_plus_de_20_salaries',
        'fnal_contribution_plus_de_50_salaries',
        'fnal_cotisation',
        # "forfait_annuel",  # Antérieur à 2011, pas présent sous forme de barème
        'formprof_11_salaries_et_plus',
        'formprof_20_salaries_et_plus',
        'formprof_entre_10_et_19_salaries',
        'formprof_moins_de_10_salaries',
        'formprof_moins_de_11_salaries',
        'maladie',
        'penibilite_additionnelle',
        'penibilite_base',
        'penibilite_multiplicateur_exposition_multiple',
        'vieillesse_deplafonnee',
        'vieillesse_plafonnee',
        ],
    'prive_non_cadre': [
        'agffnc',
        'agirc_arrco',
        'ags',
        'apprentissage_contribution_additionnelle',
        'apprentissage_taxe_alsace_moselle',
        'apprentissage_taxe',
        'arrco',
        'asf',
        'ceg',
        'cet2019',
        'chomage',
        'construction_plus_de_10_salaries',
        'construction_plus_de_20_salaries',
        'construction_plus_de_50_salaries',
        'csa',
        'famille',
        'financement_organisations_syndicales',
        'fnal_contribution_moins_de_20_salaries',
        'fnal_contribution_moins_de_50_salaries',
        'fnal_contribution_plus_de_10_salaries',
        'fnal_contribution_plus_de_20_salaries',
        'fnal_contribution_plus_de_50_salaries',
        'fnal_cotisation',
        'formprof_11_salaries_et_plus',
        'formprof_20_salaries_et_plus',
        'formprof_entre_10_et_19_salaries',
        'formprof_moins_de_10_salaries',
        'formprof_moins_de_11_salaries',
        'maladie',
        'penibilite_additionnelle',
        'penibilite_base',
        'penibilite_multiplicateur_exposition_multiple',
        'vieillesse_deplafonnee',
        'vieillesse_plafonnee',
        ],
    'public_non_titulaire': [
        'csa',
        'famille',
        'financement_organisations_syndicales',
        'fnal_contribution_moins_de_20_salaries',
        'fnal_contribution_moins_de_50_salaries',
        'fnal_contribution_plus_de_10_salaries',
        'fnal_contribution_plus_de_20_salaries',
        'fnal_contribution_plus_de_50_salaries',
        'fnal_cotisation',
        'ircantec',
        'maladie',
        'penibilite_additionnelle',
        'penibilite_base',
        'penibilite_multiplicateur_exposition_multiple',
        'vieillesse_deplafonnee',
        'vieillesse_plafonnee',
        ],
    'public_titulaire_etat': [
        'ati',
        'csa',
        'famille',
        'financement_organisations_syndicales',
        'fnal_contribution_moins_de_20_salaries',
        'fnal_contribution_moins_de_50_salaries',
        'fnal_contribution_plus_de_10_salaries',
        'fnal_contribution_plus_de_20_salaries',
        'fnal_contribution_plus_de_50_salaries',
        'fnal_cotisation',
        'maladie',
        'penibilite_additionnelle',
        'penibilite_base',
        'penibilite_multiplicateur_exposition_multiple',
        'pension',
        'rafp',
        ],
    'public_titulaire_hospitaliere': [
        'atiacl',
        'cnracl',
        'csa',
        'famille',
        'feh',
        'financement_organisations_syndicales',
        'fnal_contribution_moins_de_20_salaries',
        'fnal_contribution_moins_de_50_salaries',
        'fnal_contribution_plus_de_10_salaries',
        'fnal_contribution_plus_de_20_salaries',
        'fnal_contribution_plus_de_50_salaries',
        'fnal_cotisation',
        'maladie',
        'penibilite_additionnelle',
        'penibilite_base',
        'penibilite_multiplicateur_exposition_multiple',
        'rafp',
        ],
    'public_titulaire_militaire': [
        'cnracl',
        'csa',
        'famille',
        'financement_organisations_syndicales',
        'fnal_contribution_moins_de_20_salaries',
        'fnal_contribution_moins_de_50_salaries',
        'fnal_contribution_plus_de_10_salaries',
        'fnal_contribution_plus_de_20_salaries',
        'fnal_contribution_plus_de_50_salaries',
        'fnal_cotisation',
        'maladie',
        'penibilite_additionnelle',
        'penibilite_base',
        'penibilite_multiplicateur_exposition_multiple',
        'pension',
        'rafp',
        ],
    'public_titulaire_territoriale': [
        'atiacl',
        'cnracl',
        'csa',
        'famille',
        'fcppa',
        'financement_organisations_syndicales',
        'fnal_contribution_moins_de_20_salaries',
        'fnal_contribution_moins_de_50_salaries',
        'fnal_contribution_plus_de_10_salaries',
        'fnal_contribution_plus_de_20_salaries',
        'fnal_contribution_plus_de_50_salaries',
        'fnal_cotisation',
        'maladie',
        'penibilite_additionnelle',
        'penibilite_base',
        'penibilite_multiplicateur_exposition_multiple',
        'rafp',
        ],
    }


cotisations_salarie_by_categorie_salarie = {
    'prive_cadre': [
        'agff',
        'agirc_arrco',
        'agirc',
        'apec',
        'arrco',
        'asf',
        'ceg',
        'cet',
        'cet2019',
        'chomage',
        # "forfait_annuel",  # Antérieur à 2011, pas présent sous forme de barème
        'maladie_alsace_moselle',
        'maladie',
        'vieillesse_deplafonnee',
        'vieillesse_plafonnee',
        ],
    'prive_non_cadre': [
        'agff',
        'agirc_arrco',
        'asf',
        'arrco',
        'ceg',
        'cet2019',
        'chomage',
        'maladie_alsace_moselle',
        'maladie',
        'vieillesse_deplafonnee',
        'vieillesse_plafonnee',
        ],
    'public_non_titulaire': [
        'excep_solidarite',
        'ircantec',
        'maladie_alsace_moselle',
        'maladie',
        'vieillesse_deplafonnee',
        'vieillesse_plafonnee',
        ],
    'public_titulaire_etat': [
        'excep_solidarite',
        'pension',
        'rafp',
        ],
    'public_titulaire_hospitaliere': [
        'cnracl_s_ti',
        'cnracl_s_nbi',
        'excep_solidarite',
        'rafp',
        ],
    'public_titulaire_territoriale': [
        'cnracl_s_ti',
        'cnracl_s_nbi',
        'excep_solidarite',
        'rafp',
        ],
    }


# TODO: Rename bareme_by_type_sal_name to bareme_by_categorie_salarie


def apply_bareme_for_relevant_type_sal(
        bareme_by_type_sal_name,
        bareme_name,
        categorie_salarie,
        base,
        plafond_securite_sociale,
        round_base_decimals = DEFAULT_ROUND_BASE_DECIMALS,
        ):
    '''Apply bareme corresponding to bareme_name to the relevant categorie_salarie.'''
    assert bareme_by_type_sal_name is not None
    assert bareme_name is not None
    assert categorie_salarie is not None
    assert base is not None
    assert plafond_securite_sociale is not None
    TypesCategorieSalarie = categorie_salarie.possible_values

    def iter_cotisations():
        for categorie_salarie_type in TypesCategorieSalarie:
            if categorie_salarie_type == TypesCategorieSalarie.non_pertinent:
                continue

            if bareme_by_type_sal_name._name == 'cotisations_employeur_after_preprocessing':
                cotisations_by_categorie_salarie = cotisations_employeur_by_categorie_salarie
            elif bareme_by_type_sal_name._name == 'cotisations_salarie_after_preprocessing':
                cotisations_by_categorie_salarie = cotisations_salarie_by_categorie_salarie
            else:
                NameError()

            try:
                categorie_salarie_baremes = bareme_by_type_sal_name[categorie_salarie_type.name]
            except KeyError as e:
                # FIXME: dirty fix since public_titulaire_militaire does not exist
                if categorie_salarie_type.name == 'public_titulaire_militaire':
                    continue
                raise(e)

            if bareme_name in cotisations_by_categorie_salarie[categorie_salarie_type.name]:
                bareme = categorie_salarie_baremes[bareme_name]
            else:
                KeyError(f'{bareme_name} not in {bareme_by_type_sal_name._name} for {categorie_salarie_type.name}')
                continue

            yield bareme.calc(
                base * (categorie_salarie == categorie_salarie_type),
                factor = plafond_securite_sociale,
                round_base_decimals = round_base_decimals,
                )

    return - sum(iter_cotisations())


def apply_bareme(individu, period, parameters, cotisation_type = None, bareme_name = None, variable_name = None):
    cotisation_mode_recouvrement = individu('cotisation_sociale_mode_recouvrement', period)
    TypesCotisationSocialeModeRecouvrement = cotisation_mode_recouvrement.possible_values
    cotisation = (
        # anticipé (mensuel avec recouvrement en fin d'année)
        cotisation_mode_recouvrement == TypesCotisationSocialeModeRecouvrement.mensuel) * (
            compute_cotisation_anticipee(
                individu,
                period,
                parameters,
                cotisation_type = cotisation_type,
                bareme_name = bareme_name,
                variable_name = variable_name,
                )
            ) + (
        # en fin d'année
        cotisation_mode_recouvrement == TypesCotisationSocialeModeRecouvrement.annuel) * (
            compute_cotisation_annuelle(
                individu,
                period,
                parameters,
                cotisation_type = cotisation_type,
                bareme_name = bareme_name,
                )
            ) + (
        # mensuel strict
        cotisation_mode_recouvrement == TypesCotisationSocialeModeRecouvrement.mensuel_strict) * (
            compute_cotisation(
                individu,
                period,
                parameters,
                cotisation_type = cotisation_type,
                bareme_name = bareme_name,
                )
            )
    return cotisation


def compute_cotisation(individu, period, parameters, cotisation_type = None, bareme_name = None):
    assert cotisation_type is not None
    if cotisation_type == 'employeur':
        bareme_by_type_sal_name = parameters(period).cotsoc.cotisations_employeur
    elif cotisation_type == 'salarie':
        bareme_by_type_sal_name = parameters(period).cotsoc.cotisations_salarie
    assert bareme_name is not None

    assiette_cotisations_sociales = individu('assiette_cotisations_sociales', period, options = [ADD])
    plafond_securite_sociale = individu('plafond_securite_sociale', period, options = [ADD])
    categorie_salarie = individu('categorie_salarie', period.first_month)

    cotisation = apply_bareme_for_relevant_type_sal(
        bareme_by_type_sal_name = bareme_by_type_sal_name,
        bareme_name = bareme_name,
        base = assiette_cotisations_sociales,
        plafond_securite_sociale = plafond_securite_sociale,
        categorie_salarie = categorie_salarie,
        )
    return cotisation


def compute_cotisation_annuelle(individu, period, parameters, cotisation_type = None, bareme_name = None):
    if period.start.month < 12:
        return 0
    if period.start.month == 12:
        return compute_cotisation(
            individu,
            period.this_year,
            parameters,
            cotisation_type = cotisation_type,
            bareme_name = bareme_name,
            )


def compute_cotisation_anticipee(individu, period, parameters, cotisation_type = None, bareme_name = None, variable_name = None):
    if period.start.month < 12:
        return compute_cotisation(
            individu,
            period.first_month,
            parameters,
            cotisation_type = cotisation_type,
            bareme_name = bareme_name,
            )
    if period.start.month == 12:
        cumul = individu(variable_name, period.start.offset('first-of', 'month').offset(
            -11, 'month').period('month', 11), options = [ADD])
        # December variable_name depends on variable_name in the past 11 months.
        # We need to explicitely allow this recursion.

        return compute_cotisation(
            individu,
            period.this_year,
            parameters,
            cotisation_type = cotisation_type,
            bareme_name = bareme_name,
            ) - cumul



from numpy import floor, logical_and as and_, logical_or as or_

from openfisca_france.model.base import *
from openfisca_france.model.prestations.prestations_familiales.base_ressource import nb_enf


class api(Variable):
    value_type = float
    entity = Famille
    label = 'Allocation de parent isolé'
    reference = 'http://fr.wikipedia.org/wiki/Allocation_de_parent_isol%C3%A9',
    end = '2009-05-31'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    calculate_output = calculate_output_add

    def formula(famille, period, parameters):
        '''
        Allocation de parent isolé
        '''
        isole = not_(famille('en_couple', period))
        rsa_forfait_logement = famille('rsa_forfait_logement', period)
        rsa_base_ressources = famille('rsa_base_ressources', period)
        af_majoration = famille('af_majoration', period)
        rsa = famille('rsa', period)
        af = parameters(period).prestations_sociales.prestations_familiales.prestations_generales.af
        bmaf = parameters(period).prestations_sociales.prestations_familiales.bmaf.bmaf
        api = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.api

        # TODO:
        #    Majoration pour isolement
        #    Si vous êtes parent isolé, c’est-à-dire célibataire, divorcé(e), séparé(e) ou veuf(ve) avec des enfants
        #    à charge ou enceinte, le montant forfaitaire garanti est majoré.
        #    Ce montant forfaitaire majoré est accordé à partir du mois au cours duquel survient l'un des événements
        #    suivants :
        #    - déclaration de grossesse,
        #    - naissance d'un enfant,
        #    - prise en charge d'un enfant,
        #    - séparation, veuvage,
        #    - dépôt de la demande si l’événement est antérieur.
        #
        #    Le montant forfaitaire majoré peut être accordé pendant 12 mois, continus ou discontinus, au cours
        #    d’une période de 18 mois suivant l’événement.
        #    Si votre plus jeune enfant à charge a moins de 3 ans, le montant forfaitaire majoré vous est accordé
        #    jusqu'à ses 3 ans.
        age_en_mois_i = famille.members('age_en_mois', period)
        age_en_mois_enfant = famille.min(age_en_mois_i, role = Famille.ENFANT)
        enceinte = (age_en_mois_enfant < 0) * (age_en_mois_enfant > -6)
        # TODO: quel mois mettre ?
        # TODO: pas complètement exact
        # L'allocataire perçoit l'API :
        # jusqu'à ce que le plus jeune enfant ait 3 ans,
        # ou pendant 12 mois consécutifs si les enfants sont âgés de plus de 3 ans
        #    et s'il a présenté sa demande dans les 6 mois à partir du moment où il
        #    assure seul la charge de l'enfant.
        # TODO: API courte gens pour les gens qui ont divorcés dans l'année
        # Le droit à l'allocation est réétudié tous les 3 mois.
        # # Calcul de l'année et mois de naissance du benjamin

        condition = (floor(age_en_mois_enfant / 12) <= api.api_cond.age_limite - 1)
        eligib = isole * ((enceinte != 0) | (nb_enf(famille, period, 0, api.api_cond.age_limite - 1) > 0)) * condition

        # moins de 20 ans avant inclusion dans rsa
        # moins de 25 ans après inclusion dans rsa
        api1 = eligib * bmaf * (api.api_m.femmes_enceintes_sans_enfant_a_charge + api.api_m.supplement_par_enfant * nb_enf(famille, period, af.af_cm.age1, api.api_cond.age_pac - 1))
        rsa = (api.api_cond.age_pac >= 25)  # dummy passage au rsa majoré
        br_api = rsa_base_ressources + af_majoration * not_(rsa)
        # On pourrait mensualiser RMI, BRrmi et forfait logement
        api = max_(0, api1 - rsa_forfait_logement / 12 - br_api / 12 - rsa / 12)
        # L'API est exonérée de CRDS
        return api  # annualisé
        # TODO API: temps partiel qui modifie la base ressource
        # Cumul
        # Cumul avec un revenu
        # Si l'allocataire reprend une activité ou suit une formation professionnelle rémunérée, les revenus sont
        # cumulables intégralement au cours des 3 premiers mois de reprise d'activité.
        # Du 4e au 12e mois qui suit, le montant de l'allocation varie en fonction de la durée de l'activité
        # ou de la formation.
        # Durée d'activité de 78 heures ou plus par mois ou activité non salariée
        # Lorsque la durée d'activité est de 78 heures minimum par mois, le montant de l'API perçu par l'allocataire est
        # diminué de la totalité du salaire. Tous les revenus d'activité sont pris en compte pour le calcul de l'API,
        # sauf si l'allocataire perçoit des revenus issus d'un contrat insertion-revenu minimum d'activité (CIRMA)
        # ou d'un contrat d'avenir (CAV).
        # L'allocataire peut bénéficier, sous certaines conditions :
        # • de la prime de retour à l'emploi si son activité est d'une durée d'au moins 4 mois consécutifs,
        # sauf s'il effectue un stage de formation professionnelle,
        # • de la prime forfaitaire pendant 9 mois, sauf s'il exerce une activité salariée dans le cadre d'un CIRMA
        # ou d'un CAV.
        # Durée d'activité de moins de 78 heures par mois
        # Lorsque la durée d'activité est inférieure à 78 heures par mois, le montant de l'API perçu par l'allocataire
        # est diminué de la moitié du salaire.
        # Si l'allocataire exerce une activité dans le cadre d'un CIRMA ou d'un CAV, ses revenus d'activité ne sont pas
        # pris en compte pour le calcul de son API.


class psa(Variable):
    value_type = float
    entity = Famille
    label = 'Prime de solidarité active'
    end = '2009-04-30'
    reference = 'http://www.service-public.fr/actualites/001077.html'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    calculate_output = calculate_output_add

    def formula_2009_04(famille, period, parameters):
        '''
        Prime de solidarité active (exceptionnelle, 200€ versés une fois en avril 2009)
        Versement en avril 2009 d’une prime de solidarité active (Psa) aux familles modestes qui ont bénéficié
        en janvier, février ou mars 2009 du Rmi, de l’Api (du Rsa expérimental, du Cav ou du Rma pour
        les ex-bénéficiaires du Rmi ou de l’Api), de la prime forfaitaire mensuelle au titre du Rmi ou de l’Api
        ou enfin d’une aide au logement (à condition d’exercer une activité professionnelle et
        d’être âgé de plus de 25 ans ou d’avoir au moins un enfant à charge).
        La Psa, prime exceptionnelle, s’élève à 200 euros par foyer bénéficiaire.
        '''
        rmi = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.rmi
        api = famille('api', period)
        rsa = famille('rsa', period)
        af_nbenf = famille('af_nbenf', period)
        aide_logement = famille('aide_logement', period)

        personne_en_activite_i = (famille.members('activite', period) == TypesActivite.actif)
        parent_en_activite = famille.any(personne_en_activite_i, role = Famille.PARENT)

        dummy_api = api > 0
        dummy_rmi = rsa > 0
        dummy_al = and_(aide_logement > 0, or_(af_nbenf > 0, parent_en_activite))
        condition = (dummy_api + dummy_rmi + dummy_al > 0)
        psa = condition * rmi.psa
        return psa


class rmi(Variable):
    value_type = float
    entity = Famille
    label = "Revenu Minimum d'Insertion"
    end = '2009-05-31'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_1988_12(famille, period):
        activite_i = famille.members('activite', period)
        condition_activite_i = (
            (activite_i != TypesActivite.actif)
            * (activite_i != TypesActivite.etudiant)
            * (activite_i != TypesActivite.retraite)
            )
        condition_activite = famille.any(condition_activite_i)

        rsa_base_ressources = famille('rsa_base_ressources', period)
        rsa_socle = famille('rsa_socle', period)
        rsa_forfait_logement = famille('rsa_forfait_logement', period)

        return condition_activite * max_(0, rsa_socle - rsa_forfait_logement - rsa_base_ressources)
        # TODO: Migré lors de la mensualisation. Probablement faux


class rsa_activite(Variable):
    value_type = float
    entity = Famille
    label = 'Revenu de solidarité active - activité'
    end = '2015-12-31'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula_2009_06_01(famille, period):
        rsa = famille('rsa', period)
        rsa_base_ressources = famille('rsa_base_ressources', period)
        rsa_socle = famille('rsa_socle', period)
        rsa_forfait_logement = famille('rsa_forfait_logement', period)
        rmi = max_(0, rsa_socle - rsa_forfait_logement - rsa_base_ressources)
        return max_(rsa - rmi, 0)


class rsa_activite_individu(Variable):
    value_type = float
    entity = Individu
    label = "Revenu de solidarité active - activité au niveau de l'individu"
    end = '2015-12-31'
    definition_period = YEAR

    def formula_2009_06_01(individu, period):
        '''
        Note: le partage en moitié est un point de législation, pas un choix arbitraire
        '''
        janvier = period.first_month

        rsa_activite = individu.famille('rsa_activite', period, options = [ADD])
        marie = individu('statut_marital', janvier) == 1
        en_couple = individu.famille('en_couple', janvier)

        # On partage le rsa_activite entre les parents. Si la personne est mariée et qu'aucun conjoint n'a été déclaré,
        # on divise par 2.
        partage_rsa = or_(marie, en_couple)

        return where(partage_rsa, rsa_activite / 2, rsa_activite)



from numpy import logical_not as not_
from openfisca_france.model.base import *
from openfisca_france.model.prestations.prestations_familiales.base_ressource import nb_enf


class ars(Variable):
    value_type = float
    entity = Famille
    label = 'Allocation de rentrée scolaire'
    reference = 'http://vosdroits.service-public.fr/particuliers/F1878.xhtml'
    definition_period = YEAR

    def formula(famille, period, parameters):
        '''
        Allocation de rentrée scolaire brute de CRDS
        '''
        janvier = period.first_month
        octobre = period.start.offset('first-of', 'year').offset(9, 'month').period('month')
        decembre = period.start.offset('first-of', 'year').offset(11, 'month').period('month')
        af_nbenf = famille('af_nbenf', octobre)
        base_ressources = famille('prestations_familiales_base_ressources', janvier)
        ars = parameters(octobre).prestations_sociales.prestations_familiales.education_presence_parentale.ars
        # TODO: convention sur la mensualisation

        bmaf = parameters(octobre).prestations_sociales.prestations_familiales.bmaf.bmaf

        # Condition sur l'âge
        # Art. R543-2 :
        # "Ouvre droit à l'allocation de rentrée scolaire chaque enfant à charge qui atteindra son sixième anniversaire avant le
        # 1er février de l'année suivant celle de la rentrée scolaire."
        # "L'allocation reste due, lors de chaque rentrée scolaire, pour tout enfant qui n'a pas atteint l'âge de dix-huit ans révolus au 15 septembre de l'année considérée."
        # "Le montant de l'allocation de rentrée scolaire est majoré, d'une part, lorsque l'enfant atteint ses onze ans, et, d'autre part, lorsque l'enfant atteint ses quinze ans, au cours de l'année civile de la rentrée scolaire."
        # Ce que l'on fait :
        #   - On donne éligibilité à l'ARS aux enfants à partir de ceux ayant 6 ans au 31/12. On n'ajoute pas ceux atteingant
        #     6 ans en janvier N+1 car normalement, n'entrent en CP que les enfants atteignant 6 ans l'année de la rentrée. D'ailleurs, le site de la Cnaf évoque l'âge de six ans au 31/12 : https://www.caf.fr/allocataires/droits-et-prestations/s-informer-sur-les-aides/enfance-et-jeunesse/l-allocation-de-rentree-scolaire-ars
        #   - Les majorations dépendent de l'âge au 31/12. Donc, au total, on détermine l'éligibilité des enfants à l'ARS et leur catégorie en termes de montant en fonction de leur âge en décembre.
        #   - Exception : la fin de l'éligibilité à l'ARS intervient après 17 ans, lorsque cet âge est dépassé non pas au 31/12, mais au 15/09 de l'année de la rentrée scolaire. Condition moins restrictive. On ajoute les enfants dus à cette moindre restriction, sans prendre en compte la subtilité du 15/09 (on inclut les enfants n'ayant pas atteint 18 ans au 30/09)
        enf_primaire = nb_enf(famille, decembre, ars.ars_cond.age_entree_primaire, ars.ars_cond.age_entree_college - 1)
        enf_college = nb_enf(famille, decembre, ars.ars_cond.age_entree_college, ars.ars_cond.age_entree_lycee - 1)
        enf_lycee_moins_18_ans_decembre = nb_enf(famille, decembre, ars.ars_cond.age_entree_lycee, ars.ars_cond.age_sortie_lycee - 1)
        age_en_mois_decembre_i = famille.members('age_en_mois', decembre)
        autonomie_financiere_i = famille.members('autonomie_financiere', decembre)
        enf_lycee_eligible_18_ans_decembre = famille.sum(
            ((age_en_mois_decembre_i <= 12 * ars.ars_cond.age_sortie_lycee + 2) * (age_en_mois_decembre_i >= 12 * ars.ars_cond.age_sortie_lycee) * not_(autonomie_financiere_i)),
            role = Famille.ENFANT
            )
        enf_lycee = enf_lycee_moins_18_ans_decembre + enf_lycee_eligible_18_ans_decembre

        # Plafond en fonction du nb d'enfants A CHARGE (Cf. article R543)
        ars_plaf_res = ars.ars_plaf.plafond_ressources * (1 + af_nbenf * ars.ars_plaf.majoration_par_enf_supp)

        arsbase = bmaf * (
            ars.ars_m.taux_primaire * enf_primaire
            + ars.ars_m.taux_college * enf_college
            + ars.ars_m.taux_lycee * enf_lycee
            )

        # Montant de l'ARS, avec ARS différentielle si les ressources sont supérieures au plafond (voir art. R543-6-1 du CSS)
        ars_montant = max_(0, arsbase - max_(0, (base_ressources - ars_plaf_res)))

        return ars_montant * (ars_montant >= ars.montant_minimum_verse)



from openfisca_france.model.base import Individu, Variable, min_, max_, not_, MONTH, set_input_dispatch_by_period, set_input_divide_by_period
from numpy import ceil


class livret_epargne_populaire_plafond(Variable):
    value_type = float
    entity = Individu
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'Article L221-15 du code monétaire et financier',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=5ADBC3914320B781FBF13FBA821C392E.tplgfr25s_3?idArticle=LEGIARTI000028447486&cidTexte=LEGITEXT000006072026&dateTexte=20190515',
        'Article 1417 du Code général des impôts',
        'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=E596249E4FBBDB2A6D14A1D46EEB9451.tplgfr25s_3?idArticle=LEGIARTI000036443079&cidTexte=LEGITEXT000006069577&categorieLien=id&dateTexte=20200101'
        ]

    def formula(individu, period, parameters):
        params = parameters(period).taxation_capital.epargne.livret_epargne_populaire
        residence = individu.menage('residence', period)
        bareme = params.baremes[residence]

        nbptr = individu.foyer_fiscal('nbptr', period.n_2)
        plafond = min_(1, nbptr) * bareme.valeur_de_base + 2 * (min_(max_(0, nbptr - 1), 0.5) * bareme.premiere_demi_part + max_(0, nbptr - 1.5) * bareme.demi_part_supplementaire)

        coef = params.coefficient_multiplicateur

        return ceil(coef * plafond)


class livret_epargne_populaire_eligibilite(Variable):
    value_type = bool
    entity = Individu
    label = "Eligibilité au livret d'épargne populaire"
    reference = 'https://www.service-public.fr/particuliers/vosdroits/F2367'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        rfr = individu.foyer_fiscal('rfr', period.n_2)
        plafond = individu('livret_epargne_populaire_plafond', period)
        independent = not_(individu('enfant_a_charge', period.this_year))

        return independent * (rfr <= plafond)


class livret_epargne_populaire_taux(Variable):
    value_type = float
    entity = Individu
    label = "Taux du livret d'épargne populaire"
    reference = [
        "3° I de l'article 1 de l'arrêté du 27 janvier 2021 relatif aux taux d'intérêt des produits d'épargne réglementée",
        'https://www.legifrance.gouv.fr/loda/id/JORFTEXT000043114027/2022-07-31/',
        ]
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        eligibilite = individu('livret_epargne_populaire_eligibilite', period)

        epargne = parameters(period).taxation_capital.epargne
        base_livret_a = epargne.livret_a.taux
        majoration = epargne.livret_a.majoration_base
        taux_inflation = epargne.taux_inflation

        taux_lep = max(base_livret_a + majoration, taux_inflation)
        return eligibilite * 100 * taux_lep



import os

from openfisca_france.model.base import *

from .. import entities


dir_path = os.path.join(os.path.dirname(__file__), 'parameters')


def modify_parameters(parameters):
    file_path = os.path.join(dir_path, 'cesthra_invalidite.yaml')
    reform_parameters_subtree = load_parameter_file(name='cesthra', file_path=file_path)
    parameters.add_child('cesthra', reform_parameters_subtree)
    return parameters


class cesthra(Variable):
    value_type = float
    entity = entities.FoyerFiscal
    label = "Contribution exceptionnelle de solidarité sur les très hauts revenus d'activité"
    definition_period = YEAR
    # PLF 2013 (rejeté) : 'taxe à 75%'

    def formula(foyer_fiscal, period, parameters):
        salaire_imposable_i = foyer_fiscal.members('salaire_imposable', period, options = [ADD])
        _cesthra = parameters(period).cesthra

        cesthra_i = max_(salaire_imposable_i - _cesthra.seuil, 0) * _cesthra.taux

        return foyer_fiscal.sum(cesthra_i)


class irpp(Variable):
    label = 'Impôt sur le revenu des personnes physiques (réformée pour intégrer la cesthra)'
    definition_period = YEAR

    def formula(foyer_fiscal, period, parameters):
        '''
        Montant après seuil de recouvrement (hors ppe)
        '''
        iai = foyer_fiscal('iai', period)
        credits_impot = foyer_fiscal('credits_impot', period)
        acomptes_ir = foyer_fiscal('acomptes_ir', period)
        contribution_exceptionnelle_hauts_revenus = foyer_fiscal('contribution_exceptionnelle_hauts_revenus', period)
        cesthra = foyer_fiscal('cesthra', period = period)
        recouvrement = parameters(period).impot_revenu.calcul_impot_revenu.recouvrement

        pre_result = iai - credits_impot - acomptes_ir + contribution_exceptionnelle_hauts_revenus + cesthra

        return (
            (iai > recouvrement.seuil)
            * ((pre_result < recouvrement.min) * (pre_result > 0) * iai * 0
            + ((pre_result <= 0) + (pre_result >= recouvrement.min)) * (- pre_result))
            + (iai <= recouvrement.seuil) * ((pre_result < 0) * (-pre_result)
            + (pre_result >= 0) * 0 * iai)
            )


class cesthra_invalidee(Reform):
    name = "Contribution execptionnelle sur les très hauts revenus d'activité (invalidée par le CC)"

    def apply(self):
        self.add_variable(cesthra)
        self.update_variable(irpp)
        self.modify_parameters(modifier_function = modify_parameters)



from openfisca_france.model.base import Variable, Individu, TypesActivite, MONTH, set_input_dispatch_by_period, set_input_divide_by_period


class aide_permis_demandeur_emploi_eligibilite_financiere(Variable):
    value_type = bool
    entity = Individu
    label = "Éligibilité financière à l'aide à l’obtention du permis de conduire automobile pour les demandeurs d’emploi"
    reference = [
        'Délibération Pôle Emploi n°2011/13 du 11 avril 2011',
        'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n201113-du-11-avril.html?type=dossiers/2011/bope-n2011-36-du-18-avril-2011',
        'Bulletin officiel de Pôle emploi n°2020-05 du 17 janvier 2020',
        'http://www.bo-pole-emploi.org/bulletinsofficiels/instruction-pe-n2011-205-du-9-decembre-2011--mise-a-jour-bope-n2020-05.html?type=dossiers/2020/bope-n2020-005-du-17-janvier-2020',
        ]
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        sans_rsa = individu.famille('rsa', period) <= 0
        sans_aah = individu('aah', period) <= 0
        sans_retraite = individu('retraite_brute', period) <= 0

        allocation_journaliere_minimum = parameters(period).chomage.allocation_retour_emploi.montant_minimum_hors_mayotte
        plafond_chomage = allocation_journaliere_minimum * 31
        chomage_minimum = individu('chomage_net', period) <= plafond_chomage

        return sans_rsa * sans_aah * sans_retraite * chomage_minimum


class aide_permis_demandeur_emploi_eligibilite_individu(Variable):
    value_type = bool
    entity = Individu
    label = "Éligibilité individuelle à l'aide à l’obtention du permis de conduire automobile pour les demandeurs d’emploi"
    reference = [
        'Délibération Pôle Emploi n°2011/13 du 11 avril 2011',
        'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n201113-du-11-avril.html?type=dossiers/2011/bope-n2011-36-du-18-avril-2011'
        ]
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period):
        return individu('activite', period) == TypesActivite.chomeur


class aide_permis_demandeur_emploi(Variable):
    value_type = float
    entity = Individu
    label = 'Aide à l’obtention du permis de conduire automobile pour les demandeurs d’emploi'
    reference = [
        'Délibération Pôle Emploi n°2011/13 du 11 avril 2011',
        'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n201113-du-11-avril.html?type=dossiers/2011/bope-n2011-36-du-18-avril-2011',
        'Instruction Pôle Emploi n°2011-205 du 9 décembre 2011',
        'http://www.bo-pole-emploi.org/bulletinsofficiels/instruction-pe-n2011-205-du-9-de.html?type=dossiers/2011/bope-n2011-112-du-9-decembre-201',
        ]
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        eligibilite_financiere = individu('aide_permis_demandeur_emploi_eligibilite_financiere', period)
        eligibilite_individu = individu('aide_permis_demandeur_emploi_eligibilite_individu', period)

        montant = parameters(period).prestations_sociales.transport.aide_permis_demandeur_emploi.montant_maximum
        return montant * eligibilite_financiere * eligibilite_individu



from numpy import fabs, timedelta64

from openfisca_france.model.base import Famille, Individu, Variable, Enum, MONTH, ADD,\
    set_input_dispatch_by_period, set_input_divide_by_period, date, min_, not_
from openfisca_france.model.revenus.activite.salarie import TypesContrat, TypesLieuEmploiFormation,\
    TypesCategoriesDemandeurEmploi


class agepi_nbenf(Variable):
    value_type = int
    entity = Famille
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    label = "Nombre d'enfants pour le calcul de l'aide à la garde des enfants de parents isolés de Pôle Emploi - AGEPI"
    reference = [
        'Article 2 de la délibération n°2013-46 du 18 décembre 2013 du Pôle Emploi',
        'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n2013-46-du-18-dece.html?type=dossiers/2013/bope-n2013-128-du-24-decembre-20'
        ]

    def formula_2014_01_20(famille, period, parameters):
        age_membres_famille = famille.members('age', period)
        parametres_agepi = parameters(period).prestations_sociales.prestations_familiales.education_presence_parentale.agepi
        age_eligibles = (age_membres_famille < parametres_agepi.age_enfant_maximum) * (age_membres_famille > 0)
        nb_enfants_eligibles = famille.sum(age_eligibles, role=Famille.ENFANT)

        return nb_enfants_eligibles


class agepi_temps_travail_en_heure(Variable):
    value_type = float
    entity = Individu
    label = "Temps de travail en heures pour le calcul de l'aide à la garde des enfants de parents isolés de Pôle Emploi - AGEPI"
    definition_period = MONTH
    set_input = set_input_divide_by_period


class TypesIntensiteActivite(Enum):
    __order__ = 'inconnue hebdomadaire mensuelle'  # Needed to preserve the enum order in Python 2
    inconnue = "Intensité d'emploi ou de formation inconnue"
    hebdomadaire = "Intensité d'emploi ou de formation hebdomadaire"
    mensuelle = "Intensité d'emploi ou de formation mensuelle"


class type_intensite_activite(Variable):
    value_type = Enum
    possible_values = TypesIntensiteActivite
    default_value = TypesIntensiteActivite.inconnue
    entity = Individu
    label = "Le type d'intensité d'activité au regard de Pôle Emploi"
    definition_period = MONTH
    documentation = '''
    L'intensité d'activité est évaluée en fonction de son type (hebdomadaire, ...)
    et du nombre d'heures requises.
    '''


class agepi_date_demande(Variable):
    value_type = date
    default_value = date(1, 1, 1)
    entity = Individu
    label = "Date de demande d'évaluation à l'éligibilité à l'AGEPI (date du fait générateur)"
    definition_period = MONTH
    set_input = set_input_dispatch_by_period
    reference = 'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n2013-46-du-18-dece.html?type=dossiers/2013/bope-n2013-128-du-24-decembre-20'


class agepi_eligible(Variable):
    value_type = bool
    entity = Individu
    label = "Eligibilité à l'aide à la garde des enfants de parents isolés de Pôle Emploi - AGEPI"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'Article 4 de la délibération n°2013-46 du 18 décembre 2013 du Pôle Emploi',
        'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n2013-46-du-18-dece.html?type=dossiers/2013/bope-n2013-128-du-24-decembre-20'
        ]

    def formula_2014_01_20(individu, period, parameters):
        # L'individu élève seul son enfant (parent isolé)
        parents_isoles = individu.famille('nb_parents', period) == 1

        # L'âge du ou des enfants dont il a la garde est inférieur à 10 ans (condition de garde d'enfant)
        condition_nb_enfants = individu.famille('agepi_nbenf', period) > 0

        # L'individu n'a pas touché l'AGEPI dans les 12 derniers mois (condition de durée entre faits générateurs)
        annee_glissante = period.start.period('year').offset(-1).offset(-1, 'month')
        agepi_non_percues = not_(individu('agepi', annee_glissante, options=[ADD]))

        # L'individu est inscrit en catégorie 1, 2, 3, 4 "stagiaire de la formation professionnelle" ou 5 "contrat aidé"
        pe_categorie_demandeur_emploi = individu('pole_emploi_categorie_demandeur_emploi', period)

        stagiaire_formation_professionnelle = individu('stagiaire', period)
        contrat_aide = individu('contrat_aide', period)

        categorie_4 = pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_4
        categorie_4_stagiaire_formation_professionnelle = categorie_4 * stagiaire_formation_professionnelle

        categorie_5 = pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_5
        categorie_5_contrat_aide = categorie_5 * contrat_aide

        categories_eligibles = ((pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_1)
                                + (pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_2)
                                + (pe_categorie_demandeur_emploi == TypesCategoriesDemandeurEmploi.categorie_3)
                                + (categorie_4_stagiaire_formation_professionnelle + categorie_5_contrat_aide))

        # L'emploi ou la formation se situe en France
        lieux_activite_eligibles = not_(individu('lieu_emploi_ou_formation', period) == TypesLieuEmploiFormation.non_renseigne)

        # L'individu effectue sa demande au plus tard dans le mois qui suit sa reprise d'emploi ou de formation
        contrat_de_travail_debut = individu('contrat_de_travail_debut', period)  # numpy.datetime64
        contrat_de_travail_debut_en_mois = contrat_de_travail_debut.astype('M8[M]')

        date_demande_limite = min_(
            (contrat_de_travail_debut_en_mois + 1) + (contrat_de_travail_debut - contrat_de_travail_debut_en_mois),
            (contrat_de_travail_debut_en_mois + 2) - timedelta64(1, 'D')
            )

        agepi_date_de_demande = individu('agepi_date_demande', period)
        dates_demandes_agepi_eligibles = agepi_date_de_demande <= date_demande_limite

        # L'individu est non indemnisé ou son ARE est inférieure ou égale à l'ARE minimale
        mayotte = individu.menage('residence_mayotte', period)
        hors_mayotte = not_(mayotte)

        allocation_individu = individu('allocation_retour_emploi_journaliere', period)

        parametres_are = parameters(period).chomage.allocation_retour_emploi
        allocation_minimale_hors_mayotte = parametres_are.montant_minimum_hors_mayotte * hors_mayotte
        allocation_minimale_mayotte = parametres_are.montant_minimum_mayotte * mayotte

        allocation_minimale_en_fonction_de_la_region = allocation_minimale_hors_mayotte + allocation_minimale_mayotte

        # Montant ARE minimum en fonction de la région (Mayotte / hors Mayotte)
        # (et diminution de la précision car une comparaison : 14.77 <= 14.77 renvoyait un False)
        epsilon = 0.0001
        are_individu_egale_are_min = fabs(allocation_individu - allocation_minimale_en_fonction_de_la_region) < epsilon
        are_individu_inferieure_are_min = allocation_individu < allocation_minimale_en_fonction_de_la_region

        montants_are_eligibles = are_individu_inferieure_are_min + are_individu_egale_are_min

        # L'individu est en reprise d'emploi du type CDI, CDD ou CTT d'au moins 3 mois consécutifs ou en processus d'entrée en formation d'une durée supérieure ou égale à 40 heures
        reprises_types_activites = individu('contrat_de_travail_type', period)

        reprises_types_activites_formation = reprises_types_activites == TypesContrat.formation
        reprises_types_activites_cdi = reprises_types_activites == TypesContrat.cdi
        reprises_types_activites_cdd = reprises_types_activites == TypesContrat.cdd
        reprises_types_activites_ctt = reprises_types_activites == TypesContrat.ctt

        # La formation doit être supérieure ou égale à 40 heures
        duree_formation = individu('duree_formation', period)
        parametres_agepi = parameters(period).prestations_sociales.prestations_familiales.education_presence_parentale.agepi
        periode_formation_eligible = duree_formation >= parametres_agepi.duree_de_formation_minimum

        # Le durée de contrat de l'emploi doit être d'au moins 3 mois
        periode_de_contrat_3_mois_minimum = individu('contrat_de_travail_duree', period) >= parametres_agepi.duree_cdd_ctt_minimum

        reprises_types_activites_formation_eligible = reprises_types_activites_formation * periode_formation_eligible
        reprises_types_activites_cdd_eligible = reprises_types_activites_cdd * periode_de_contrat_3_mois_minimum
        reprises_types_activites_ctt_eligible = reprises_types_activites_ctt * periode_de_contrat_3_mois_minimum

        types_et_duree_activite_eligibles = (reprises_types_activites_formation_eligible
                                            + reprises_types_activites_cdi
                                            + reprises_types_activites_cdd_eligible
                                            + reprises_types_activites_ctt_eligible)

        eligible_agepi = (parents_isoles
            * condition_nb_enfants
            * agepi_non_percues
            * categories_eligibles
            * lieux_activite_eligibles
            * dates_demandes_agepi_eligibles
            * montants_are_eligibles
            * types_et_duree_activite_eligibles)

        return eligible_agepi


class agepi_hors_mayotte(Variable):
    value_type = float
    entity = Individu
    label = "Montant de l'aide à la garde des enfants de parents isolés de Pôle Emploi - AGEPI - Cas HORS MAYOTTE"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'Article 4 de la délibération n°2013-46 du 18 décembre 2013 du Pôle Emploi',
        'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n2013-46-du-18-dece.html?type=dossiers/2013/bope-n2013-128-du-24-decembre-20',
        '2. Aide à la garde d’enfants pour les parents isolés (AGEPI)'
        ]

    def formula_2014_01_20(individu, period, parameters):
        est_parent = individu.has_role(Famille.PARENT)
        intensite_activite = individu('type_intensite_activite', period)
        nb_heures = individu('agepi_temps_travail_en_heure', period)
        nb_enfants_eligibles = individu.famille('agepi_nbenf', period)
        eligibilite_agepi = individu('agepi_eligible', period)

        intensite_hebdomadaire = intensite_activite == TypesIntensiteActivite.hebdomadaire
        intensite_mensuelle = intensite_activite == TypesIntensiteActivite.mensuelle

        hors_mayotte = not_(individu.menage('residence_mayotte', period))

        parametres_agepi = parameters(period).prestations_sociales.prestations_familiales.education_presence_parentale.agepi
        parametres_montants = parametres_agepi.montants.hors_mayotte
        montants_min_hors_mayotte = parametres_montants.minimum.calc(nb_enfants_eligibles)
        montants_max_hors_mayotte = parametres_montants.maximum.calc(nb_enfants_eligibles)
        intensite_hebdo_seuil = parametres_agepi.intensite_hebdomadaire_seuil
        intensite_mensuelle_seuil = parametres_agepi.intensite_mensuelle_seuil

        montants_min_intensite = montants_min_hors_mayotte * (intensite_hebdomadaire + intensite_mensuelle)
        montants_max_intensite = montants_max_hors_mayotte * (intensite_hebdomadaire + intensite_mensuelle)

        condition_montants_min = ((nb_heures < intensite_hebdo_seuil) * intensite_hebdomadaire) + ((nb_heures < intensite_mensuelle_seuil) * intensite_mensuelle)
        condition_montants_max = ((nb_heures >= intensite_hebdo_seuil) * intensite_hebdomadaire) + ((nb_heures >= intensite_mensuelle_seuil) * intensite_mensuelle)

        montant_avec_intensite = (condition_montants_min * montants_min_intensite) + (condition_montants_max * montants_max_intensite)

        montants = hors_mayotte * (est_parent * montant_avec_intensite)

        return eligibilite_agepi * montants


class agepi_mayotte(Variable):
    value_type = float
    entity = Individu
    label = "Montant de l'aide à la garde des enfants de parents isolés de Pôle Emploi - AGEPI - Cas MAYOTTE"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'Article 4 de la délibération n°2013-46 du 18 décembre 2013 du Pôle Emploi',
        'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n2013-46-du-18-dece.html?type=dossiers/2013/bope-n2013-128-du-24-decembre-20',
        '2. Aide à la garde d’enfants pour les parents isolés (AGEPI)',
        'http://www.bo-pole-emploi.org/bulletinsofficiels/instruction-dg-n2014-48-du-6-jui.html?type=dossiers/2014/bope-n2014-62-du-18-juin-2014',
        ]

    def formula_2014_01_20(individu, period, parameters):
        est_parent = individu.has_role(Famille.PARENT)
        intensite_activite = individu('type_intensite_activite', period)
        nb_heures = individu('agepi_temps_travail_en_heure', period)
        nb_enfants_eligibles = individu.famille('agepi_nbenf', period)
        eligibilite_agepi = individu('agepi_eligible', period)

        intensite_hebdomadaire = intensite_activite == TypesIntensiteActivite.hebdomadaire
        intensite_mensuelle = intensite_activite == TypesIntensiteActivite.mensuelle

        mayotte = individu.menage('residence_mayotte', period)

        parametres_agepi = parameters(period).prestations_sociales.prestations_familiales.education_presence_parentale.agepi
        parametres_montants = parametres_agepi.montants.mayotte
        montants_min_mayotte = parametres_montants.minimum.calc(nb_enfants_eligibles)
        montants_max_mayotte = parametres_montants.maximum.calc(nb_enfants_eligibles)
        intensite_hebdo_seuil = parametres_agepi.intensite_hebdomadaire_seuil
        intensite_mensuelle_seuil = parametres_agepi.intensite_mensuelle_seuil

        montants_min_intensite = montants_min_mayotte * (intensite_hebdomadaire + intensite_mensuelle)
        montants_max_intensite = montants_max_mayotte * (intensite_hebdomadaire + intensite_mensuelle)

        condition_montants_min = ((nb_heures < intensite_hebdo_seuil) * intensite_hebdomadaire) + ((nb_heures < intensite_mensuelle_seuil) * intensite_mensuelle)
        condition_montants_max = ((nb_heures >= intensite_hebdo_seuil) * intensite_hebdomadaire) + ((nb_heures >= intensite_mensuelle_seuil) * intensite_mensuelle)

        montant_avec_intensite = (condition_montants_min * montants_min_intensite) + (condition_montants_max * montants_max_intensite)

        montants = mayotte * (est_parent * montant_avec_intensite)

        return eligibilite_agepi * montants


class agepi(Variable):
    value_type = float
    entity = Individu
    label = "Montant de l'aide à la garde des enfants de parents isolés de Pôle Emploi - AGEPI"
    definition_period = MONTH
    set_input = set_input_divide_by_period
    reference = [
        'Article 4 de la délibération n°2013-46 du 18 décembre 2013 du Pôle Emploi',
        'http://www.bo-pole-emploi.org/bulletinsofficiels/deliberation-n2013-46-du-18-dece.html?type=dossiers/2013/bope-n2013-128-du-24-decembre-20',
        '2. Aide à la garde d’enfants pour les parents isolés (AGEPI)',
        'http://www.bo-pole-emploi.org/bulletinsofficiels/instruction-dg-n2014-48-du-6-jui.html?type=dossiers/2014/bope-n2014-62-du-18-juin-2014',
        ]

    def formula_2014_01_20(individu, period):

        agepi_mayotte = individu('agepi_mayotte', period)
        agepi_hors_mayotte = individu('agepi_hors_mayotte', period)

        return agepi_hors_mayotte + agepi_mayotte



from typing import Callable, Tuple

import numpy
import pytest

from ..cache import tax_benefit_system
from openfisca_core.simulations import Simulation
from openfisca_france.reforms.cesthra_invalidee import cesthra_invalidee


@pytest.fixture
def simulations(new_scenario) -> Callable[..., Tuple[Simulation]]:
    def _simulations(year: int) -> Tuple[Simulation]:
        scenario = new_scenario(
            reform = cesthra_invalidee(tax_benefit_system),
            count = 10,
            _max = 30000,
            _min = 0,
            name = 'salaire_imposable',
            year = year,
            people = 4,
            )

        return scenario.new_simulation(use_baseline = True), scenario.new_simulation()

    return _simulations


def test_cesthra_impots_directs(simulations, year = 2012):
    actual_simulation, reform_simulation = simulations(year)
    actual_impots = actual_simulation.calculate('impots_directs', year)
    reform_impots = reform_simulation.calculate('impots_directs', year)
    assert numpy.all(numpy.equal(actual_impots, reform_impots))


def test_cesthra_revdisp(simulations, year = 2012):
    actual_simulation, reform_simulation = simulations(year)
    actual_revdisp = actual_simulation.calculate('revenu_disponible', year)
    reform_revdisp = reform_simulation.calculate('revenu_disponible', year)
    assert numpy.all(numpy.equal(actual_revdisp, reform_revdisp))




from openfisca_core.parameters import ParameterNode, Scale
from openfisca_france import FranceTaxBenefitSystem


tax_benefit_system = FranceTaxBenefitSystem()
parameters = tax_benefit_system.parameters


def get_parameters_by_unit(parameter, parameters_by_unit = None):
    '''
    Build a dictionnary collecting the legislation parameters according to their units
    '''
    if parameters_by_unit is None:
        parameters_by_unit = dict(
            scale_none = list(),
            scale_currency = list(),
            none = list(),
            currency = list(),
            rate = list(),
            year = list(),
            )

    for sub_parameter in parameter.children.values():
        if isinstance(sub_parameter, ParameterNode):
            get_parameters_by_unit(sub_parameter, parameters_by_unit)
        else:
            if isinstance(sub_parameter, Scale):
                unit = sub_parameter.metadata.get('unit')
                rate_unit = sub_parameter.metadata.get('rate_unit')
                threshold_unit = sub_parameter.metadata.get('threshold_unit')
                if unit is not None:
                    raise ValueError('Scale {} should not have a unit = {}'.format(
                        sub_parameter.name, unit))
                elif (rate_unit is None) and (threshold_unit is None):
                    parameters_by_unit['scale_none'].append(sub_parameter)
                elif threshold_unit == 'currency':
                    parameters_by_unit['scale_currency'].append(sub_parameter)
                else:
                    raise ValueError('Scale {} has a stange threshold_unit = {}, rate_unit = {}'.format(
                        sub_parameter.name, threshold_unit, rate_unit))

                continue

            unit = sub_parameter.metadata.get('unit')

            if unit is None:
                parameters_by_unit['none'].append(sub_parameter)
            elif unit == '/1':
                parameters_by_unit['rate'].append(sub_parameter)
            elif unit == 'currency':
                parameters_by_unit['currency'].append(sub_parameter)
            elif unit == 'year':
                parameters_by_unit['year'].append(sub_parameter)
            else:
                raise ValueError('Parameter {} has a stange unit {}'.format(
                    sub_parameter.name, unit))

    return parameters_by_unit


if __name__ == '__main__':
    import logging

    logging.basicConfig(level = logging.INFO)
    logger = logging.getLogger(__name__)

    parameters_by_unit = get_parameters_by_unit(parameters)

    logger.info('Distribution of parameters types:')

    for type_, sub_parameters in parameters_by_unit.items():
        logger.info(type_, len(sub_parameters))

    logger.info('\n')
    logger.info('List of parameters with no units')

    for param in parameters_by_unit['none']:
        logger.info(param.name)

    logger.info('\n')
    logger.info('List of scale with currency')

    for param in parameters_by_unit['scale_currency']:
        logger.info(param.name)

    logger.info('\n')
    logger.info('List of scale no unit')

    for param in parameters_by_unit['scale_none']:
        logger.info(param.name)



from openfisca_france.model.base import *


class rente_accident_travail(Variable):
    value_type = float
    entity = Individu
    label = 'Montant mensuel de la rente d’accident du travail'
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006073189&idArticle=LEGIARTI000006743072&dateTexte=&categorieLien=cid'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        previous_year = period.start.period('year').offset(-1)
        non_salarie_agricole = individu('rpns_benefice_exploitant_agricole', previous_year, options=[ADD]) != 0
        rente_accident_travail_salarie = individu('rente_accident_travail_salarie', period)
        rente_accident_travail_exploitant_agricole = individu('rente_accident_travail_exploitant_agricole', period)

        return where(non_salarie_agricole, rente_accident_travail_exploitant_agricole, rente_accident_travail_salarie)


class rente_accident_travail_salarie(Variable):
    value_type = float
    entity = Individu
    label = 'Montant de la rente d’accident du travail pour les victimes salariées'
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006073189&idArticle=LEGIARTI000006743072&dateTexte=&categorieLien=cid'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        previous_year = period.start.period('year').offset(-1)
        salarie = individu('salaire_net', previous_year, options=[ADD]) != 0
        rente_accident_travail_rachat = individu('rente_accident_travail_rachat', period)
        taux_incapacite = individu('taux_accident_travail', period)
        rente_accident_travail_base = individu('rente_accident_travail_base', period) * salarie
        rente_accident_travail_apres_rachat = individu('rente_accident_travail_apres_rachat', period)

        montant_rente_accident_travail = where(rente_accident_travail_rachat != 0, rente_accident_travail_apres_rachat,
                                               rente_accident_travail_base)

        return select(
            [taux_incapacite < 0.1, taux_incapacite >= 0.1],
            [0, montant_rente_accident_travail / 12]
            )


class rente_accident_travail_exploitant_agricole(Variable):
    value_type = float
    entity = Individu
    label = "Montant de la rente d’accident du travail pour les chefs d'exploitation ou d'entreprise agricole"
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006071367&idArticle=LEGIARTI000006598097&dateTexte=&categorieLien=cid'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period):
        previous_year = period.start.period('year').offset(-1)
        non_salarie_agricole = individu('rpns_benefice_exploitant_agricole', previous_year, options=[ADD]) != 0
        rente_accident_travail_rachat = individu('rente_accident_travail_rachat', period)
        taux_incapacite = individu('taux_accident_travail', period)
        rente_accident_travail_base = individu('rente_accident_travail_base', period) * non_salarie_agricole
        rente_accident_travail_apres_rachat = individu('rente_accident_travail_apres_rachat', period)

        montant_rente_accident_travail = where(rente_accident_travail_rachat != 0, rente_accident_travail_apres_rachat,
                                               rente_accident_travail_base)
        return select(
            [taux_incapacite < 0.3, taux_incapacite >= 0.3],
            [0, montant_rente_accident_travail / 12]
            )


class indemnite_accident_travail(Variable):
    value_type = float
    entity = Individu
    label = "Indemnité selon le taux d'incapacité"
    reference = 'https://www.legifrance.gouv.fr/affichCode.do?idSectionTA=LEGISCTA000006172216&cidTexte=LEGITEXT000006073189'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        indem_at = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.accident_travail.rente.taux
        taux_incapacite = individu('taux_accident_travail', period)

        return indem_at.indemnite_accident_travail.calc(taux_incapacite * 100)


class rente_accident_travail_base(Variable):
    value_type = float
    entity = Individu
    label = 'Montant de base de la rente d’accident du travail'
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do?cidTexte=LEGITEXT000006073189&idArticle=LEGIARTI000006743072&dateTexte=&categorieLien=cid'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        param_rente_at = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.accident_travail.rente.taux
        taux_incapacite = individu('taux_accident_travail', period)
        taux = param_rente_at.bareme.calc(taux_incapacite)
        taux_rente_accident_travail = select([taux_incapacite < param_rente_at.taux_minimum], [0], default=taux)
        rente_accident_travail_base = individu('rente_accident_travail_salaire_utile', period) * taux_rente_accident_travail

        return rente_accident_travail_base


class demande_rachat(Variable):
    value_type = bool
    entity = Individu
    label = 'La victime a demandé le rachat partiel de la rente'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period


class rente_accident_travail_apres_rachat(Variable):
    value_type = float
    entity = Individu
    label = 'Rente d’accident du travail, reliquat suite à conversion en capital'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        rente_at = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.accident_travail.rente.taux
        age = min_(max_(individu('age', period), 16), 100)
        rente_accident_travail_rachat = individu('rente_accident_travail_rachat', period)
        conversion_rente_capital = rente_at.capital_representatif[age]
        rente_accident_travail_base = individu('rente_accident_travail_base', period)
        rente_apres_rachat = rente_accident_travail_base - (rente_accident_travail_rachat / conversion_rente_capital)

        return rente_apres_rachat


class rente_accident_travail_rachat(Variable):
    value_type = float
    entity = Individu
    label = 'Rachat de la rente d’accident du travail'
    reference = 'https://www.legifrance.gouv.fr/eli/arrete/2016/12/19/AFSS1637858A/jo/texte'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        rente_at = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.accident_travail.rente.taux
        demande_rachat = individu('demande_rachat', period)
        age = min_(max_(individu('age', period), 16), 100)
        conversion_rente_capital = rente_at.capital_representatif[age]
        rente_accident_travail_base = individu('rente_accident_travail_base', period)
        rachat = (rente_accident_travail_base * conversion_rente_capital) / 4

        return rachat * demande_rachat


class pcrtp_nombre_actes_assistance(Variable):
    value_type = int
    entity = Individu
    label = "Nombre d'actes nécessitant l'assistance d'une tierce personne"
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=6D8F7F6917ADFBDEAFE1D8A432F39229.tplgfr23s_2?idArticle=LEGIARTI000027267037&cidTexte=LEGITEXT000006073189&dateTexte=20181218'
    definition_period = MONTH
    set_input = set_input_divide_by_period


class pcrtp(Variable):
    value_type = float
    entity = Individu
    label = 'Prestation complémentaire pour recours à tierce personne (PCRTP)'
    reference = 'https://www.legifrance.gouv.fr/affichCode.do?idSectionTA=LEGISCTA000006172216&cidTexte=LEGITEXT000006073189'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        rente_at = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.accident_travail.rente.taux
        taux_incapacite = individu('taux_accident_travail', period)
        pcrtp_nombre_actes_assistance = individu('pcrtp_nombre_actes_assistance', period)
        montant_pcrtp = rente_at.pcrtp[pcrtp_nombre_actes_assistance]

        return montant_pcrtp * (taux_incapacite >= 0.8)


class rente_accident_travail_salaire_utile(Variable):
    value_type = float
    entity = Individu
    label = 'Salaire utile pour calculer la rente d’accident du travail'
    reference = 'https://www.legifrance.gouv.fr/affichCodeArticle.do;jsessionid=7392B9902E4B974EAE8783FAF2D69849.tplgfr30s_1?idArticle=LEGIARTI000006750376&cidTexte=LEGITEXT000006073189&dateTexte=20180823'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        previous_year = period.start.period('year').offset(-1)
        rente_at = parameters(period).prestations_sociales.solidarite_insertion.minima_sociaux.accident_travail.rente

        salaire_net = individu('salaire_net', previous_year, options=[ADD])
        rpns_benefice_exploitant_agricole = individu('rpns_benefice_exploitant_agricole', previous_year, options=[ADD])
        salaire = max_(salaire_net, rpns_benefice_exploitant_agricole)
        salaire_net_base = max_(rente_at.salaire_net.salaire_minimum, salaire)
        coef = salaire_net_base / rente_at.salaire_net.salaire_minimum
        bareme = rente_at.salaire_net.bareme.calc(coef)
        return rente_at.salaire_net.salaire_minimum * bareme



from numpy import datetime64, timedelta64


from openfisca_france.model.base import *


class apprenti(Variable):
    value_type = bool
    entity = Individu
    label = "L'individu est apprenti"
    reference = 'http://www.apce.com/pid927/contrat-d-apprentissage.html?espace=1&tp=1&pagination=2'
    definition_period = MONTH
    set_input = set_input_dispatch_by_period

    def formula(individu, period, parameters):
        age = individu('age', period)
        age_condition = (16 <= age) * (age < 25)
        apprentissage_contrat_debut = individu('apprentissage_contrat_debut', period)
        duree_contrat = (
            datetime64(period.start) + timedelta64(1, 'D') - apprentissage_contrat_debut
            ).astype('timedelta64[Y]')
        anciennete_contrat = (duree_contrat < timedelta64(3, 'Y'))

        return age_condition * anciennete_contrat


class remuneration_apprenti(Variable):
    value_type = float
    entity = Individu
    label = "Rémunération de l'apprenti"
    reference = 'http://www.apce.com/pid927/contrat-d-apprentissage.html?espace=1&tp=1&pagination=2'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    # Aux jeunes de 16 à 25 ans (exceptionnellement 15 ans, s'ils ont effectué la scolarité du premier cycle de
    # l'enseignement secondaire, ou, s'ils suivent une "formation apprentissage junior").
    #
    # Depuis le 30 juillet 2011, il est possible pour un jeune mineur ayant 15 ans au cours de l'année civile, de
    # souscrire un contrat d'apprentissage s'il justifie avoir accompli la scolarité du premier cycle de l'enseignement
    # secondaire, ou avoir suivi une formation dans le cadre du dispositif d'initiation aux métiers en
    # alternance (DIMA).

    def formula(individu, period, parameters):
        age = individu('age', period)
        apprentissage_contrat_debut = individu('apprentissage_contrat_debut', period)
        smic = parameters(period).marche_travail.salaire_minimum.smic.smic_b_horaire * 52 * 35 / 12
        anciennete_contrat = (
            datetime64(period.start) + timedelta64(1, 'D') - apprentissage_contrat_debut
            ).astype('timedelta64[Y]')
        apprenti = individu('apprenti', period)
        salaire_en_smic = [  # TODO: move to parameters
            dict(
                part_de_smic_by_anciennete = {
                    1: .25,
                    2: .41,
                    3: .53,
                    },
                age_min = 15,
                age_max = 18,
                ),
            dict(
                part_de_smic_by_anciennete = {
                    1: .37,
                    2: .49,
                    3: .61,
                    },
                age_min = 18,
                age_max = 21,
                ),
            dict(
                part_de_smic_by_anciennete = {
                    1: .53,
                    2: .65,
                    3: .78,
                    },
                age_min = 21,
                age_max = 99
                )
            ]

        output = age * 0.0
        for age_interval in salaire_en_smic:
            age_condition = (age_interval['age_min'] <= age) * (age < age_interval['age_max'])
            output[age_condition] = sum([
                (anciennete_contrat[age_condition] == timedelta64(anciennete, 'Y')) * part_de_smic
                for anciennete, part_de_smic in age_interval['part_de_smic_by_anciennete'].items()
                ])
        return output * smic * apprenti


class exoneration_cotisations_employeur_apprenti(Variable):
    value_type = float
    entity = Individu
    label = 'Exonération de cotisations employeur pour un apprenti'
    reference = 'http://www.apce.com/pid927/contrat-d-apprentissage.html?espace=1&tp=1&pagination=2'
    definition_period = MONTH
    set_input = set_input_divide_by_period
    # Artisans et employeurs de moins de 11 salariés
    #
    # - exonération totale (part patronale et salariale) des charges sociales,
    # - sauf : cotisation sociale patronale d'accidents du travail et des maladies professionnelles, cotisation
    #   supplémentaire accidents du travail et cotisation supplémentaire de retraite complémentaire (c'est-à-dire
    #   dépassant le taux minimum obligatoire).
    #
    # Autres entreprises
    #
    # - exonération totale (part patronale et salariale) des cotisations de sécurité sociale
    #   (maladie-veuvage-vieillesse) et d'allocations familiales,
    # - exonération des autres cotisations sociales salariales,
    # - restent donc dues par l'employeur : les cotisations supplémentaires d'accidents du travail, la part patronale
    #   des cotisations d'accidents du travail et de maladies professionnelles, de retraite complémentaire, d'assurance
    #   chômage et d'AGFF, le versement transport ainsi que les cotisations Fnal.
    # Précision : le décompte de l'effectif des entreprises non artisanales s'apprécie au 31 décembre précédant la date
    # de conclusion du contrat d'apprentissage.

    def formula(individu, period, parameters):
        accident_du_travail = individu('accident_du_travail', period)
        apprenti = individu('apprenti', period)
        cotisations_employeur = individu('cotisations_employeur', period)
        effectif_entreprise = individu('effectif_entreprise', period)
        famille = individu('famille', period)
        mmid_employeur = individu('mmid_employeur', period)
        vieillesse_deplafonnee_employeur = individu('vieillesse_deplafonnee_employeur', period)
        vieillesse_plafonnee_employeur = individu('vieillesse_plafonnee_employeur', period)

        cotisations_non_exonerees = accident_du_travail
        exoneration_moins_11 = cotisations_non_exonerees - cotisations_employeur

        cotisations_exonerees = (
            famille
            + mmid_employeur
            + vieillesse_plafonnee_employeur
            + vieillesse_deplafonnee_employeur
            )

        exoneration_plus_11 = -cotisations_exonerees

        return (
            exoneration_plus_11
            * (effectif_entreprise >= 11)
            + exoneration_moins_11
            * (effectif_entreprise < 11)
            ) * apprenti


class exoneration_cotisations_salariales_apprenti(Variable):
    value_type = float
    entity = Individu
    label = "Exonération de cotisations salariales pour l'emploi d'un apprenti"
    reference = 'http://www.apce.com/pid927/contrat-d-apprentissage.html?espace=1&tp=1&pagination=2'
    definition_period = MONTH
    set_input = set_input_divide_by_period

    def formula(individu, period, parameters):
        apprenti = individu('apprenti', period)
        cotisations_salariales_contributives = individu('cotisations_salariales_contributives', period)
        cotisations_salariales_non_contributives = individu(
            'cotisations_salariales_non_contributives', period)
        return - (cotisations_salariales_contributives + cotisations_salariales_non_contributives) * apprenti


class prime_apprentissage(Variable):
    value_type = float
    entity = Individu
    label = "Prime d'apprentissage pour les entreprise employant un apprenti"
    reference = 'http://www.apce.com/pid927/contrat-d-apprentissage.html?espace=1&tp=1&pagination=2'
    definition_period = YEAR
    # L'employeur peut également recevoir de la région dans laquelle est situé l'établissement du lieu de travail,
    # une prime d'apprentissage.
    #
    # Les conditions d'attribution de cette aide sont fixées par chaque région (ou pour la Corse, par la collectivité
    # territoriale de Corse) après avis du comité de coordination régional de l'emploi et de la formation
    # professionnelle en tenant compte notamment de l'ensemble de l'effort de l'employeur dans le domaine de
    # l'apprentissage, de la durée de la formation et des objectifs de développement de la formation professionnelle
    # des jeunes sur le territoire de la région (ou de la collectivité territoriale de Corse).
    #
    # Son montant est au minimum de 1 000 euros par année de cycle de formation.
    # nouveau. Depuis le 1er janvier 2014 , cette aide n'est versée qu'aux entreprises de moins de 11 salariés.
    #
    # Son versement est subordonné à la condition que l'embauche de l'apprenti soit confirmée à l'issue des deux
    # premiers mois de l'apprentissage.
    #
    # Son versement cesse lorsque l'apprenti n'est plus salarié dans l'entreprise ou l'établissement qui l'a embauché.

    def formula(individu, period, parameters):
        apprenti = individu('apprenti', period)
        return 1000 * apprenti

# # class credit_impot_emploi_apprenti(Variable):
#     value_type = float
#     entity = Individu
#     label = " Crédit d'impôt pour l'emploi d'apprentis"
#     reference = "http://www.apce.com/pid927/contrat-d-apprentissage.html?espace=1&tp=1&pagination=2"
#
#     def formula(individu, period, parameters):
#         pass
#     # Cet avantage fiscal est réservé aux entreprises imposées selon un régime d'imposition du réel.
#     # Précision : les entreprises exonérées d'impôt sur les bénéfices au titre des entreprises nouvelles, d'une
#     # implantation en zone franche urbaine, du statut de jeune entreprise innovante ou d'une implantation en Corse
#     # peuvent également en bénéficier.
#     #
#     # Le crédit d'impôt est égal au nombre moyen d'apprentis dont le contrat de travail a atteint une durée d'au moins
#     # 1 mois au cours de l'année civile multiplié par :
#     # - 1 600 €,
#     # - ou 2 200€ si l'apprenti est reconnu travailleur handicapé et qu'il bénéficie d'un accompagnement personnalisé,
#     # ou si l'apprenti est employé par une entreprise portant le label "Entreprise du patrimoine vivant", ou s'il est
#     # recruté dans le cadre d'une "formation apprentissage junior".
#     #
#     # L'avantage fiscal est plafonné au montant des dépenses de personnel afférentes aux apprentis minoré des
#     # subventions perçues en contrepartie de leur embauche.


# # class credit_impot_emploi_apprenti(Variable):
#     value_type = float
#     entity = Individu
#     label = "Déduction de la créance "bonus alternant"
# Les entreprises de plus de 250 salariés, tous établissements confondus, redevables de la taxe d'apprentissage,
# qui emploient plus de 4 % de jeunes en apprentissage (5 % pour la taxe payable en 2016 au titre de 2015), dans la
# limite de 6 % d'alternants, peuvent bénéficier d'une créance à déduire du hors quota de la taxe d'apprentissage (TA).
# Les entreprises concernées doivent calculer elles-mêmes le montant de la créance à déduire de leur TA.
# Son montant est calculé selon la formule suivante : pourcentage d'alternants ouvrant droit à l'aide x effectif annuel
# moyen de l'entreprise au 31 décembre de l'année précédente x un montant forfaitaire de 400 € par alternant.
# Par exemple, une entreprise de 300 salariés employant 6 % de salariés en alternance, ce qui porte le nombre
# d'alternants ouvrant droit à l'aide à 2 % (6 % - 4 %), peut bénéficier d'une prime de : 2 % x 300 x 400 = 2 400 €.
